from django.db import models
from django.utils.translation import gettext as _
from datetime import datetime
from account.models import Account
from .manager import LeaveManager


class Leave(models.Model):
    SICK = 'sick'
    CASUAL = 'casual'
    EMERGENCY = 'emergency'
    STUDY = 'study'
    MATERNITY = 'maternity'
    BEREAVEMENT = 'bereavement'
    QUARANTINE = 'quarantine'
    COMPENSATORY = 'compensatory'
    SABBATICAL = 'sabbatical'

    LEAVE_TYPE = (
        (SICK, 'Sick Leave'),
        (CASUAL, 'Casual Leave'),
        (EMERGENCY, 'Emergency Leave'),
        (STUDY, 'Study Leave'),
        (MATERNITY, 'Maternity Leave'),
        (BEREAVEMENT, 'Bereavement Leave'),
        (QUARANTINE, 'Self Quarantine'),
        (COMPENSATORY, 'Compensatory Leave'),
        (SABBATICAL, 'Sabbatical Leave')
    )

    PENDING = 'pending'
    APPROVED = 'approved'
    REJECTED = 'rejected'
    CANCELLED = 'cancelled'
    COMPLETED = 'completed'
    STATUS_CHOICES = (
        (PENDING, 'Pending'),
        (APPROVED, 'Approved'),
        (REJECTED, 'Rejected'),
        (CANCELLED, 'Cancelled'),
        (COMPLETED, 'Completed'),  # Added 'Completed' status
    )
    DAYS = 30

    user = models.ForeignKey(Account, on_delete=models.CASCADE, default=1)
    startdate = models.DateField(verbose_name=_('Start Date'), help_text='leave start date is on ..', null=True,
                                 blank=False)
    enddate = models.DateField(verbose_name=_('End Date'), help_text='coming back on ...', null=True, blank=False)
    leavetype = models.CharField(choices=LEAVE_TYPE, max_length=25, default=SICK, null=True, blank=False)
    reason = models.CharField(verbose_name=_('Reason for Leave'), max_length=255,
                              help_text='add additional information for leave', null=True, blank=True)
    defaultdays = models.PositiveIntegerField(verbose_name=_('Leave days per year counter'), default=DAYS, null=True,
                                              blank=True)
    status = models.CharField(max_length=12, choices=STATUS_CHOICES, default=PENDING)
    is_approved = models.BooleanField(default=False)
    updated = models.DateTimeField(auto_now=True)
    created = models.DateTimeField(auto_now_add=True)

    objects = LeaveManager()

    class Meta:
        verbose_name = _('Leave')
        verbose_name_plural = _('Leaves')
        ordering = ['-created']

    def __str__(self):
        return f"{self.leavetype} - {self.user}"

    @property
    def pretty_leave(self):
        employee = self.user.employee_set.first().get_full_name
        return f"{employee} - {self.leavetype}"

    @property
    def leave_days(self):
        if self.startdate and self.enddate and self.startdate <= self.enddate:
            return (self.enddate - self.startdate).days
        return None

    @property
    def leave_approved(self):
        return self.is_approved

    @property
    def approve_leave(self):
        if not self.is_approved:
            self.is_approved = True
            self.status = self.APPROVED
            self.save()

    @property
    def unapprove_leave(self):
        if self.is_approved:
            self.is_approved = False
            self.status = self.PENDING
            self.save()

    @property
    def leaves_cancel(self):
        self.is_approved = False
        self.status = self.CANCELLED
        self.save()

    @property
    def reject_leave(self):
        self.is_approved = False
        self.status = self.REJECTED
        self.save()

    @property
    def is_rejected(self):
        return self.status == self.REJECTED
